% This function is part of the reproducible research addendum of the research
% paper:
% T. Necciari, N. Holighaus, P. Balazs, Z. Prusa, P. Majdak, and O. Derrien
% "Audlet Filter Banks: A Versatile Analysis/Synthesis Framework using Auditory Frequency Scales"
%
% This file along with the auxiliary file
% - femvocd1.wav
% generates the binary masks for the targeted singing voice signal. Given
% the parameters \beta and thr, a mask is created for each filter prototype.
% The masks are returned as parameters for use in the main function 
% 'hann_gtfb_separation_exp2.m'
%
% To run this file, you will need the LTFAT Toolbox V.2.2.1 or later,
% available at: http://ltfat.github.io
% 
% Author: T.Necciari 

function [MASKH,MASKGT,MASKGAB] = gen_bmask_voice(beta,thr,fmin,fmax,V)

[s,sr] = wavload('femvocd1.wav'); % Original voice source
L = length(s);
Ls = ceil(L/sr)*sr;
s = [s;zeros(Ls-L,1)];

% Set filters' spacing
spac = 1/V;

% Generate filters with Hann prototype
[audlets,a,fc,~] = audfilters(sr,Ls,'fractional','spacing',spac,'bwmul',beta,'fmin',fmin,'fmax',fmax);
% Generate filters with compactly-supported gammatone prototype
[gt,~,~,~] = audfilters(sr,Ls,'fractional','gammatone','spacing',spac,'bwmul',beta,'fmin',fmin,'fmax',fmax);
% Compute parameters for the equivalent Gabor transform
M = 2048;
a_gab = 256;% i.e. 12,5% overlap

% figure(1)
c_aud_voc_org = filterbank(s,audlets,a);
MASK1 = plotfilterbank(c_aud_voc_org,a,fc,sr,60,'audtick');
title(['Singing voice (original source) analyzed by Audlet FB with Hann filters, $\beta$ = ' num2str(beta)], 'interpreter','latex')

MASKH = zeros(size(MASK1));
ind = MASK1>thr;
MASKH(ind) = MASK1(ind)+abs(thr);

% figure(2)
gt_erb_voc_org = filterbank(s,gt,a);
MASK2 = plotfilterbank(gt_erb_voc_org,a,fc,sr,60,'audtick');
title(['Singing voice (original source) analyzed by gammatone FB, $\beta$ = ' num2str(beta)], 'interpreter','latex')
% colormap(1-gray)
% print -deps targetvoiceGFB.eps

MASKGT = zeros(size(MASK2));
ind = MASK2>thr;
MASKGT(ind) = MASK2(ind)+abs(thr);

% figure(3)
c_dgt = dgtreal(s,{'hann',M},a_gab,M);
MASK3 = plotdgtreal(c_dgt,a_gab,M,sr,60);
title(['Singing voice (original source) analyzed by Gabor transform, $\beta$ = ' num2str(beta)], 'interpreter','latex')

MASKGAB = zeros(size(MASK3));
ind = MASK3>thr;
MASKGAB(ind) = MASK3(ind)+abs(thr);

close all
clear MASK1 MASK2 MASK3

% Display masks
% figure(4)
% imagesc(MASK1)
% title(['Mask for Hann prototype, $\beta$ = ' num2str(beta)], 'interpreter','latex');
% 
% figure(5)
% imagesc(MASK2)
% title(['Mask for gammatone prototype, $\beta$ = ' num2str(beta)], 'interpreter','latex');
% 
% figure(6)
% imagesc(MASK3)
% title(['Mask for Gabor transform, $\beta$ = ' num2str(beta)], 'interpreter','latex');

% eof